"use strict";
const models = require("./../models/index");
import commonHelper from "../helpers/common";
const { getEpoch, documentUploadToS3 } = new commonHelper();
import { HttpCodes } from "../helpers/responseCodes";
const libre = require('libreoffice-convert');

export default class DocumentServices {
    constructor() {
        this.insertDocument = this.insertDocument.bind(this);
        this.GetDocuments = this.GetDocuments.bind(this);
        this.DocToPdfConvert = this.DocToPdfConvert.bind(this);
        this.DeleteDocument = this.DeleteDocument.bind(this);
    }
    //insert all document
    async insertDocument(req, callback) {
        try {
            const user_id = req.user;
            const from_language_code = req?.body?.from_language_code;
            const to_language_code = req?.body?.to_language_code;
            const application_type = parseInt(req.body.application_type);
            const title = req?.body?.title;
            if (application_type !== 2) {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'InvalidApplicationType', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
            const bucketName = process.env.S3_BUCKET_NAME;
            const folderName = 'documents/';
            let originalDocumentValue = null;
            let translatedDocumentValue = null;
            let originalFile = null;
            let translatedFile = null;
            // Find the original and translated files based on field names
            for (const file of req.files) {
                if (file.fieldname === 'document_file') {
                    originalFile = file;
                } else if (file.fieldname === 'translate_document') {
                    translatedFile = file;
                }
            }
            if (originalFile && translatedFile) {
                const uploadOriginalDocument = await documentUploadToS3(originalFile.originalname, bucketName, folderName, originalFile.buffer);
                const uploadTranslatedDocument = await documentUploadToS3(translatedFile.originalname, bucketName, folderName, translatedFile.buffer);
                if (!uploadOriginalDocument || !uploadTranslatedDocument) {
                    return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'UploadProperDocument', code: HttpCodes['BAD_REQUEST'], data: [] });
                }
                originalDocumentValue = uploadOriginalDocument.data;
                translatedDocumentValue = uploadTranslatedDocument.data;
            } else {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'BothDocumentsRequired', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
            let insertDocument = await models.Documents.create({
                user_id: user_id,
                title: title,
                original_document: originalDocumentValue,
                translate_document: translatedDocumentValue,
                from_language_code: from_language_code,
                to_language_code: to_language_code,
                created_date: await getEpoch(),
            })
            if (insertDocument) {
                return callback(null, { status: HttpCodes['API_SUCCESS'], msg: 'DocumentUpload', code: HttpCodes.OK, data: insertDocument });
            }
            else {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'SomeThingWentWrong', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
        } catch (error) {
            console.log(error, 'error.....');
            return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'SomeThingWentWrong', code: HttpCodes['BAD_REQUEST'], data: [] });
        }
    }
    //get  all documnets as per user
    async GetDocuments(req, callback) {
        try {
            const orderBy = 'sort_order';
            const order = 'ASC';
            const application_type = parseInt(req.query.application_type);
            if (application_type !== 2) {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'InvalidApplicationType', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
            const getDocumentData = await models.Documents.findAll({
                attributes: ['document_id', 'title', 'original_document', 'translate_document', 'from_language_code', 'to_language_code', 'created_date'],
                where: { user_id: req.user },
                raw: true,
            });
            if (getDocumentData.length > 0) {
                const Documents = getDocumentData.map((doc) => ({
                    document_id: doc.document_id,
                    title: doc.title || '',
                    original_document: doc.original_document,
                    translate_document: doc.translate_document,
                    from_language_code: doc.from_language_code,
                    to_language_code: doc.to_language_code,
                    created_date: doc.created_date,
                }));
                return callback(null, { 'status': HttpCodes['API_SUCCESS'], 'msg': 'ListDocuments', code: HttpCodes['OK'], data: Documents });
            } else {
                return callback(null, { 'status': HttpCodes['API_SUCCESS'], 'msg': 'NoDocumentFound', code: HttpCodes['OK'], data: [] });
            }
        } catch (err) {
            console.error('Error fetching advertisement data:', err);
            return callback(null, { 'status': HttpCodes['API_FAILURE'], 'msg': 'ErrorFetchingData', code: HttpCodes['BAD_REQUEST'], data: [] });
        }
    }
    //convert doc to pdf 
    async DocToPdfConvert(req, callback) {
        try {
            const docFile = req.file;
            if (!docFile || !docFile.buffer) {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'Invalid file object or file buffer is missing.', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
            // Validate the file type
            if (docFile.mimetype !== 'application/msword'
                && docFile.mimetype !== 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
                && docFile.mimetype !== 'application/wps-office.doc'
                && docFile.mimetype !== 'application/docx') {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'Invalid file type. Only Word documents (.doc) are allowed.', code: HttpCodes['BAD_REQUEST'], data: {} });
            }
            const bucketName = process.env.S3_BUCKET_NAME;
            const folderName = 'documents/';
            const pdfFileName = docFile.originalname.replace(/\.[^/.]+$/, '') + '.pdf';
            const docBuffer = req.file.buffer; // Use req.file.buffer as the file buffer
            // const outputFilePath = path.join(__dirname, '');
            libre.convert(docBuffer, '.pdf', undefined, async (err, result) => {
                if (err) {
                    console.error('Conversion error:', err);
                    return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'ConversionError', code: HttpCodes['BAD_REQUEST'], data: {} });
                } else {
                    // const uploadTranslatedDocument = await documentUploadToS3(pdfFileName, bucketName, folderName, result);
                    // if (!uploadTranslatedDocument) {
                    //     return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'UploadProperDocument', code: HttpCodes['BAD_REQUEST'], data: [] });
                    // }
                    // Send the binary response with the appropriate headers
                    return callback(null, {
                        status: HttpCodes['API_SUCCESS'], msg: 'DocumentUpload', code: HttpCodes.OK,
                        // data: uploadTranslatedDocument.data,
                        data: result
                        // headers: {
                        //     'Content-Type': 'application/pdf',
                        //     'Content-Disposition': 'inline; filename=output-sample.pdf', // Set the filename as needed
                        // },
                    });
                }
            });
        } catch (error) {
            console.log(error, 'error.....');
            return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'SomeThingWentWrong', code: HttpCodes['BAD_REQUEST'], data: {} });
        }
    }
    // delete document
    async DeleteDocument(req, callback) {
        try {
            const document_id = req.body.document_id;
            const user_id = req.body.user_id;
            const application_type = parseInt(req.body.application_type);
            // Check if application type is valid or not
            if (application_type !== 2) {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'InvalidApplicationType', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
            const getData = await models.Documents.findOne({
                where: { document_id: document_id }
            });
            if (!getData) {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'documentDoesNotExist', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
            const removeDocInfo = await models.Documents.destroy({
                where: { document_id: document_id }
            });
            if (removeDocInfo) {
                return callback(null, { status: HttpCodes['API_SUCCESS'], msg: 'DocDeletedSuccessfully', code: HttpCodes.OK, data: [] });
            } else {
                return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'SomeThingWentWrong', code: HttpCodes['BAD_REQUEST'], data: [] });
            }
        } catch (error) {
            console.log(error, 'error.....');
            return callback(null, { status: HttpCodes['API_FAILURE'], msg: 'SomeThingWentWrong', code: HttpCodes['BAD_REQUEST'], data: [] });
        }
    }
}